Rem
* Example 007 Collision

In this tutorial, I will show how To detect Collisions with the Irrlicht Engine.
I will describe 3 methods: Automatic collision detection For moving through 3d
worlds with stair climbing And sliding, manual triangle picking, And manual
scene node picking.

To start, we take the program from tutorial 2, which loads And displays a quake
3 level. We will use the level To walk in it And To pick triangles from it. In
addition we'll place 3 animated models into it for scene node picking. The
following code starts up the engine And loads a quake 3 level. I will Not
explain it, because it should already be known from tutorial 2.
EndRem
SuperStrict
Framework Irrlicht.Core

' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local device:IrrlichtDevice = ..
	IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480), 16, False)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

device.getFileSystem().addZipFileArchive("../../media/map-20kdm2.pk3")

Local q3levelmesh:IAnimatedMesh = smgr.getMesh("20kdm2.bsp")

Local q3node:ISceneNode
If (q3levelmesh) Then q3node = smgr.addOctTreeSceneNode(q3levelmesh.getMesh(0))

Rem
So far so good, we've loaded the quake 3 level like in tutorial 2. Now,
here comes something different: We Create a triangle selector. A
triangle selector is a class which can fetch the triangles from scene
nodes For doing different things with them, For example collision
detection. There are different triangle selectors, And all can be
created with the ISceneManager. In this example, we Create an
OctTreeTriangleSelector, which optimizes the triangle output a little
bit by reducing it like an octree. This is very useful For huge meshes
like quake 3 levels. After we created the triangle selector, we attach
it To the q3node. This is Not necessary, but in this way, we do Not
need To care For the selector, For example dropping it after we do Not
need it anymore.
EndRem

Local selector:ITriangleSelector
	
If (q3node)
	q3node.setPosition(_VECTOR3DF(-1350,-130,-1400))

	selector = smgr.createOctTreeTriangleSelector( ..
		q3levelmesh.getMesh(0), q3node, 128)
	q3node.setTriangleSelector(selector)
EndIf


Rem
We add a first person shooter camera To the scene For being able To
move in the quake 3 level like in tutorial 2. But this, time, we add a
special animator To the camera: A Collision Response animator. This
animator modifies the scene node To which it is attached To in order To
prevent it moving through walls, And To add gravity To it. The
only thing we have To tell the animator is how the world looks like,
how big the scene node is, how much gravity To apply And so on. After the 
collision response animator is attached To the camera, we do Not have To do
anything more For collision detection, anything is done automatically,
all other collision detection code below is For picking. And please
note another cool feature: The collision response animator can be
attached also To all other scene nodes, Not only To cameras. And it can
be mixed with other scene node animators. In this way, collision
detection And response in the Irrlicht engine is really, really easy.

Now we'll take a closer look on the parameters of
createCollisionResponseAnimator(). The first parameter is the
TriangleSelector, which specifies how the world, against collision
detection is done looks like. The second parameter is the scene node,
which is the Object, which is affected by collision detection, in our
Case it is the camera. The third defines how big the Object is, it is
the radius of an ellipsoid. Try it out And change the radius To smaller
values, the camera will be able To move closer To walls after this. The
Next parameter is the direction And speed of gravity.  We'll set it to 
(0, -10, 0), which approximates To realistic gravity, assuming that our
units are metres. You could set it To (0,0,0) To disable gravity. And the 
last value is just a translation: Without this, the ellipsoid with which 
collision detection is done would be around the camera, And the camera would 
be in the middle of the ellipsoid. But as human beings, we are used To have our
eyes on top of the body, with which we collide with our world, Not in
the middle of it. So we place the scene node 50 units over the center
of the ellipsoid with this parameter. And that's it, collision
detection works now.
EndRem

' Set a jump speed of 3 units per second, which gives a fairly realistic jump
' when used with the gravity of (0, -10, 0) in the collision response animator.
Local camera:ICameraSceneNode = ..
	smgr.addCameraSceneNodeFPS(Null, 100.0, .3, -1, Null, True, 3)
camera.setPosition(_VECTOR3DF(-100,50,-150))

If selector Then
	Local anim:ISceneNodeAnimator = smgr.createCollisionResponseAnimator( ..
		selector, camera, _VECTOR3DF(30,50,30), ..
		_VECTOR3DF(0,-10,0), ..
		_VECTOR3DF(0,50,0))
	camera.addAnimator(anim)
	anim.drop()
EndIf 

Rem
Because collision detection is no big deal in irrlicht, I'll describe how to
do two different types of picking in the Next section. But before this,
I'll prepare the scene a little. I need three animated characters which we
could pick later, a dynamic light For lighting them,
a billboard For drawing where we found an intersection,	And, yes, I need To
get rid of this mouse cursor. :)
EndRem

' disable mouse cursor

device.getCursorControl().setVisible(False)

' add billboard

Local bill:IBillboardSceneNode = smgr.addBillboardSceneNode()
bill.setMaterialType(EMT_TRANSPARENT_ADD_COLOR )
bill.setMaterialTexture(0, driver.getTexture("../../media/particle.bmp"))
bill.setMaterialFlag(EMF_LIGHTING, False)
bill.setMaterialFlag(EMF_ZBUFFER, False)
bill.setSize(_DIMENSION2DF(20.0, 20.0))

' add 3 animated faeries.

Local material:SMaterial=SMaterial.Create()
material.setTexture(0,driver.getTexture("../../media/faerie2.bmp"))
material.setLighting(True)

Local node:IAnimatedMeshSceneNode
Local faerie:IAnimatedMesh = smgr.getMesh("../../media/faerie.md2")

If (faerie)

	node = smgr.addAnimatedMeshSceneNode(faerie)
	node.setPosition(_VECTOR3DF(-70,0,-90))
	node.setMD2Animation(EMAT_RUN)
	node.setMaterial(0,material)

	node = smgr.addAnimatedMeshSceneNode(faerie)
	node.setPosition(_VECTOR3DF(-70,0,-30))
	node.setMD2Animation(EMAT_SALUTE)
	node.setMaterial(0,material)

	node = smgr.addAnimatedMeshSceneNode(faerie)
	node.setPosition(_VECTOR3DF(-70,0,-60))
	node.setMD2Animation(EMAT_JUMP)
	node.setMaterial(0,material)
	
EndIf

material.setTexture(0,Null)
material.setLighting(False)

' Add a light
	
smgr.addLightSceneNode(Null, _VECTOR3DF(-60,100,400), ..
	_SCOLORF(1.0,1.0,1.0,1.0), 600.0)

Rem
For Not making it To complicated, I'm doing picking inside the drawing
loop. We take two pointers For storing the current And the last
selected scene node And start the loop.
EndRem


Local selectedSceneNode:ISceneNode
Local lastSelectedSceneNode:ISceneNode


Local lastFPS:Int = -1



While(device.run())
	If (device.isWindowActive())

		driver.beginScene(True, True, Null)
	
		smgr.drawAll()
	
		Rem
		After we've drawn the whole scene with smgr.drawAll(), we'll
		do the first picking: We want To know which triangle of the
		world we are looking at. In addition, we want the exact point
		of the quake 3 level we are looking at. For this, we Create a
		3d line starting at the position of the camera And going
		through the lookAt-target of it. Then we ask the collision
		manager If this line collides with a triangle of the world
		stored in the triangle selector. If yes, we draw the 3d
		triangle And set the position of the billboard To the
		intersection point.
		EndRem
	
		Local line:Line3df = Line3df.Create()
		
		line.getStart().setEq(camera.getPosition())
		line.getEnd().setEq(line.getStart().Plus(camera.getTarget().Minus(line.getStart()).normalize().Mult(1000.0)))
	
		Local intersection:Vector3df = _VECTOR3DF()
		Local tri:Triangle3df = Triangle3df.Create()
		
		If (smgr.getSceneCollisionManager().getCollisionPoint( ..
			line, selector, intersection, tri))

			bill.setPosition(intersection)
				
			driver.SetTransform(ETS_WORLD, Matrix4.Create())
			driver.setMaterial(material)
			driver.draw3DTriangle(tri, _SCOLOR(0,255,0,0))
		EndIf
	
		Rem
		Another Type of picking supported by the Irrlicht Engine is
		scene node picking based on bounding boxes. Every scene node has
		got a bounding box, And because of that, it's very fast for
		example To get the scene node which the camera looks at. Again,
		we ask the collision manager For this, And If we've got a scene
		node, we highlight it by disabling Lighting in its material, If
		it is Not the billboard Or the quake 3 level.
		EndRem
	
		selectedSceneNode = ..
			smgr.getSceneCollisionManager().getSceneNodeFromCameraBB(camera)

		If (lastSelectedSceneNode) Then ..
			lastSelectedSceneNode.setMaterialFlag(EMF_LIGHTING, True)

		If (selectedSceneNode = q3node Or selectedSceneNode = bill) Then ..
			selectedSceneNode = Null

		If (selectedSceneNode) Then ..
			selectedSceneNode.setMaterialFlag(EMF_LIGHTING, False)

		lastSelectedSceneNode = selectedSceneNode
	
		Rem
		That's it, we just have to finish drawing.
		EndRem
	
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
		
		If (lastFPS <> fps)
			Local str:String = "Collision detection example - Irrlicht Engine ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
EndWhile
	
selector.drop()
device.drop()

