Rem
* Example 009 Mesh Viewer

This tutorial show how To Create a more complex application with the engine.
We construct a simple mesh viewer using the user interface API And the
scene management of Irrlicht. The tutorial show how To Create And use Buttons,
Windows, Toolbars, Menus, ComboBoxes, Tabcontrols, Editboxes, Images,
MessageBoxes, SkyBoxes, And how To parse XML files with the integrated XML
reader of the engine.

We start like in most other tutorials: Include all nesessary header files, add
a comment To let the engine be linked with the Right .lib file in Visual
Studio, And declare some Global variables. We also add two 'using namespace'
statements, so we do Not need To write the whole names of all classes. In this
tutorial, we use a lot stuff from the gui namespace.
EndRem
Framework Irrlicht.Core
Import BRL.FileSystem

Rem
Some Global variables used later on
EndRem

Global Device:IrrlichtDevice
Global StartUpModelFile:String
Global MessageText:String
Global Caption:String
Global Model:IAnimatedMeshSceneNode
Global SkyBox:ISceneNode
Global Octree:Int = False

Global Cameras:ICameraSceneNode[2]

Rem
Toggle between various cameras
EndRem
Function setActiveCamera ( newActive:ICameraSceneNode )
	If Not Device Then Return

	Local active:ICameraSceneNode = Device.getSceneManager().getActiveCamera()

	newActive.setInputReceiverEnabled( True )
	Device.getSceneManager().setActiveCamera ( newActive )
EndFunction

Rem
The three following functions do several stuff used by the mesh viewer. The
first Function showAboutText() simply displays a messagebox with a caption And
a message text. The texts will be stored in the MessageText And Caption
variables at startup.
EndRem
Function showAboutText()
	'  create modal message box with the text
	'  loaded from the xml file.
	Device.getGUIEnvironment().addMessageBox( ..
		Caption, MessageText)
EndFunction

Rem
The second Function loadModel() loads a model And displays it using an
addAnimatedMeshSceneNode And the scene manager. Nothing difficult. It also
displays a Short message box, If the model could Not be loaded.
EndRem
Function loadModel(filename:String)

	'  modify the name If it a .pk3 file
	Local extension:String = ExtractExt(filename)
	extension = extension.ToLower()

	'  If a texture is loaded apply it To the current model..
	If (	extension = "jpg" Or extension = "pcx" Or ..
			extension = "png" Or extension = "ppm" Or ..
			extension = "pgm" Or extension = "pbm" Or ..
			extension = "psd" Or extension = "tga" Or ..
			extension = "bmp" Or extension = "wa" ..
		)
		
		Local texture:ITexture = ..
			Device.getVideoDriver().getTexture( filename )

		If ( texture And Model )
			'  always reload texture
			Device.getVideoDriver().removeTexture ( texture )
			texture = Device.getVideoDriver().getTexture( filename )

			Model.setMaterialTexture(0, texture )
		EndIf
		
		Return
	EndIf

	'  If a archive is loaded add it To the FileSystems..
	If (extension = "pk3" Or extension = "zip")
		Device.getFileSystem().addZipFileArchive( filename )
		Return
	ElseIf (extension = ".pak")
		Device.getFileSystem().addPakFileArchive( filename )
		Return
	EndIf
	
	' Load a model into the engine

	If (Model) Then Model.remove()

	Model = Null

	Local m:IAnimatedMesh = Device.getSceneManager().getMesh( filename )

	If Not m
		'  model could Not be loaded
		If (StartUpModelFile <> filename)
			Device.getGUIEnvironment().addMessageBox( ..
				Caption, "The model could not be loaded. " + ..
				"Maybe it is not a supported file format.")
			Return
		EndIf
	EndIf

	' set Default material properties

	If (Octree)
		Model = IAnimatedMeshSceneNode(Device.getSceneManager().addOctTreeSceneNode(m.getMesh(0)))
	Else
		Local animModel:IAnimatedMeshSceneNode = Device.getSceneManager().addAnimatedMeshSceneNode(m)
		animModel.setAnimationSpeed(30)
		Model = animModel
	EndIf
	
	Model.setMaterialFlag(EMF_LIGHTING, False)
	Model.setDebugDataVisible(EDS_OFF)
	
	' we need To uncheck the menu entries. would be cool To fake a menu event, but
	' that's not so simple. so we do it brute force
	Local menu:IGUIContextMenu = IGUIContextMenu(Device.getGUIEnvironment().getRootGUIElement().getElementFromId(400, True))
	If (menu)
		menu.setItemChecked(1, False)
		menu.setItemChecked(2, False)
		menu.setItemChecked(3, False)
		menu.setItemChecked(4, False)
		menu.setItemChecked(5, False)
		menu.setItemChecked(6, False)
	EndIf
	Device.getGUIEnvironment().getRootGUIElement().getElementFromId(901, True).setText("1.0")
	Device.getGUIEnvironment().getRootGUIElement().getElementFromId(902, True).setText("1.0")
	Device.getGUIEnvironment().getRootGUIElement().getElementFromId(903, True).setText("1.0")
EndFunction


Rem
Finally, the third Function creates a toolbox window. In this simple mesh
viewer, this toolbox only contains a tab control with three edit boxes For
changing the scale of the displayed model.
EndRem
Function createToolBox()

	'  remove tool box If already there
	Local env:IGUIEnvironment = Device.getGUIEnvironment()
	Local root:IGUIElement = env.getRootGUIElement()
	Local e:IGUIElement = root.getElementFromId(5000, True)
	If (e) Then e.remove()

	'  create the toolbox window
	Local wnd:IGUIWindow = env.addWindow(_RECTI(600,45,800,480), ..
		False, "Toolset", 0, 5000)
		
	'  create tab control And tabs
	Local tab:IGUITabControl = env.addTabControl( ..
		_RECTI(2,20,800-602,480-7), wnd, True, True)

	Local t1:IGUITab = tab.addTab("Config")

	'  add some edit boxes And a button To tab one
	env.addStaticText("Scale:", ..
			_RECTI(10,20,150,45), False, False, t1)
	env.addStaticText("X:", _RECTI(22,48,40,66), False, False, t1)
	env.addEditBox("1.0", _RECTI(40,46,130,66), True, t1, 901)
	env.addStaticText("Y:", _RECTI(22,82,40,100), False, False, t1)
	env.addEditBox("1.0", _RECTI(40,76,130,96), True, t1, 902)
	env.addStaticText("Z:", _RECTI(22,108,40,126), False, False, t1)
	env.addEditBox("1.0", _RECTI(40,106,130,126), True, t1, 903)

	env.addButton(_RECTI(10,134,85,165), t1, 1101, "Set")

	' add transparency control
	env.addStaticText("GUI Transparency Control:", ..
		_RECTI(10,200,150,225), True, False, t1)
	Local scrollbar:IGUIScrollBar = ..
		env.addScrollBar(True, _RECTI(10,225,150,240), t1, 104)
	scrollbar.setMax(255)
	scrollbar.setPos(255)

	' add framerate control
	env.addStaticText("Framerate:", ..
			_RECTI(10,240,150,265), True, False, t1)
	scrollbar = env.addScrollBar(True, ..
			_RECTI(10,265,150,280), t1, 105)
	scrollbar.setMax(1000)
	scrollbar.setPos(30)

	' bring irrlicht engine logo To front, because it
	' now may be below the newly created toolbox
	'root.bringToFront(root.getElementFromId(666, True))
EndFunction


Rem
To get all the events sent by the GUI Elements, we need To Create an event
receiver. This one is really simple. If an event occurs, it checks the id of
the caller And the event Type, And starts an action based on these values. For
example, If a menu item with id 100 was selected, If opens a file-open-dialog.
EndRem
Type MyEventReceiver Extends IEventReceiver

	Method OnEvent:Int(event:SEvent)

		'  Escape swaps Camera Input
		If (event.getEventType() = EET_KEY_INPUT_EVENT And ..
			event.getKeyPressedDown() = False)

			If (event.getKeyInputKey() = EKEY_ESCAPE)
					
				If ( Device )				
					Local camera:ICameraSceneNode = ..
						Device.getSceneManager().getActiveCamera()
					If ( camera )
						camera.setInputReceiverEnabled ( Not camera.isInputReceiverEnabled() )
					EndIf
					Return True
				EndIf
			ElseIf (event.getKeyInputKey() = EKEY_F1)
				If (Device)
					Local elem:IGUIElement = Device.getGUIEnvironment().getRootGUIElement().getElementFromId(2001)
					If (elem) Then ..
						elem.setVisible(Not elem.isVisible())
				EndIf
			EndIf
		EndIf

		If (event.getEventType() = EET_GUI_EVENT)
			Local id:Int = event.getGUIEventCaller().getID()
			Local env:IGUIEnvironment = device.getGUIEnvironment()			
			Local evtype:Int=event.getGUIEventType()
			Local pos:Int 
			
			Select evtype

				Case EGET_MENU_ITEM_SELECTED

					' a menu item was clicked

					Local menu:IGUIContextMenu = IGUIContextMenu(event.getGUIEventCaller())
					id = menu.getItemCommandId(menu.getSelectedItem())

					Select id
						Case 100 ' File -> Open Model
							env.addFileOpenDialog("Please select a model file to open")
						Case 101 '  File -> Set Model Archive
							env.addFileOpenDialog("Please select your game archive/directory")
						Case 102 ' File -> LoadAsOctree
							Octree = Not Octree
							menu.setItemChecked(menu.getSelectedItem(), Octree)
						Case 200 ' File -> Quit
							Device.closeDevice()
						Case 300 ' View -> Skybox
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							SkyBox.setVisible(Not SkyBox.isVisible())
						Case 401 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem()+1, False)
							menu.setItemChecked(menu.getSelectedItem()+2, False)
							menu.setItemChecked(menu.getSelectedItem()+3, False)
							menu.setItemChecked(menu.getSelectedItem()+4, False)
							menu.setItemChecked(menu.getSelectedItem()+5, False)
							menu.setItemChecked(menu.getSelectedItem()+6, False)
							If (Model) Then ..
								Model.setDebugDataVisible(EDS_OFF)						
						Case 410 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_BBOX))							
						Case 420 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_NORMALS))
						Case 430 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_SKELETON))
						Case 440 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_MESH_WIRE_OVERLAY))
						Case 450 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_HALF_TRANSPARENCY))
						Case 460 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem(), Not menu.isItemChecked(menu.getSelectedItem()))
							If (Model) Then ..
								Model.setDebugDataVisible((Model.isDebugDataVisible() ~ EDS_BBOX_BUFFERS))
						Case 499 ' View -> Debug Information
							menu.setItemChecked(menu.getSelectedItem()-1, True)
							menu.setItemChecked(menu.getSelectedItem()-2, True)
							menu.setItemChecked(menu.getSelectedItem()-3, True)
							menu.setItemChecked(menu.getSelectedItem()-4, True)
							menu.setItemChecked(menu.getSelectedItem()-5, True)
							menu.setItemChecked(menu.getSelectedItem()-6, True)
							If (Model) Then ..
								Model.setDebugDataVisible(EDS_FULL)
						Case 500 ' Help.About
							showAboutText()
						Case 610 ' View -> Material . Solid
							If (Model) Then ..
								Model.setMaterialType(EMT_SOLID)
						Case 620 ' View -> Material . Transparent
							If (Model) Then ..
								Model.setMaterialType(EMT_TRANSPARENT_ADD_COLOR)
						Case 630 ' View -> Material . Reflection
							If (Model) Then ..
								Model.setMaterialType(EMT_SPHERE_MAP)
						Case 1000
							setActiveCamera(Cameras[0])
						Case 1100
							setActiveCamera(Cameras[1])
					EndSelect
				
			Case EGET_FILE_SELECTED
					' Load the model file, selected in the file open dialog
					Local dialog:IGUIFileOpenDialog = .. 
						IGUIFileOpenDialog(event.getGUIEventCaller())
					loadModel(dialog.getFileName())

			Case EGET_SCROLL_BAR_CHANGED

				' control skin transparency
				If id = 104
					pos = IGUIScrollBar(event.getGUIEventCaller()).getPos()
					For Local i:Int=0 Until EGDC_COUNT
						Local col:SColor = env.getSkin().GetColor(i)
						col.SetAlpha(pos)
						env.getSkin().SetColor(i, col)
					Next
				ElseIf id = 105
					pos = IGUIScrollBar(event.getGUIEventCaller()).getPos()
					If (ESNT_ANIMATED_MESH = Model.getType()) Then ..
						IAnimatedMeshSceneNode(Model).setAnimationSpeed(pos)
				EndIf

			Case EGET_COMBO_BOX_CHANGED

				' control anti-aliasing/filtering
				If id = 108
					pos = IGUIComboBox(event.getGUIEventCaller()).getSelected()
					Select pos
						Case 0
							If (Model)
								Model.setMaterialFlag(EMF_BILINEAR_FILTER, False)
								Model.setMaterialFlag(EMF_TRILINEAR_FILTER, False)
								Model.setMaterialFlag(EMF_ANISOTROPIC_FILTER, False)
							EndIf
						Case 1
							If (Model)
								Model.setMaterialFlag(EMF_BILINEAR_FILTER, True)
								Model.setMaterialFlag(EMF_TRILINEAR_FILTER, False)
							EndIf
						Case 2
							If (Model)
								Model.setMaterialFlag(EMF_BILINEAR_FILTER, False)
								Model.setMaterialFlag(EMF_TRILINEAR_FILTER, True)
							EndIf
						Case 3
							If (Model)
								Model.setMaterialFlag(EMF_ANISOTROPIC_FILTER, True)
							EndIf
						Case 4
							If (Model)
								Model.setMaterialFlag(EMF_ANISOTROPIC_FILTER, False)
							EndIf
					EndSelect
				EndIf		

			Case EGET_BUTTON_CLICKED

				Select id
					Case 1101
						' set scale
						Local root:IGUIElement = env.getRootGUIElement()
						Local scale:Vector3df=Vector3df.Create()
						Local s:String=""

						s = root.getElementFromId(901, True).getText()
						scale.setX(Float(s))
						s = root.getElementFromId(902, True).getText()
						scale.setY(Float(s))
						s = root.getElementFromId(903, True).getText()
						scale.setZ(Float(s))

						If (Model)
							Model.SetScale(scale)
						EndIf
					Case 1102
						env.addFileOpenDialog("Please select a model file to open")
					Case 1103
						showAboutText()
					Case 1104
						createToolBox()
					Case 1105
						env.addFileOpenDialog("Please select your game archive/directory")

					EndSelect
			EndSelect
		EndIf

		Return False
	EndMethod

	' we must override the generate function in order for instantiation to work properly
	Function generate:IEventReceiver()
		Return IEventReceiver(New MyEventReceiver)
	EndFunction	
EndType

Rem
Most of the hard work is done. We only need To Create the Irrlicht Engine
device And all the buttons, menus And toolbars. We start up the engine as
usual, using createDevice(). To make our application Catch events, we set our
eventreceiver as parameter. The #ifdef WIN32 preprocessor commands are Not
necessary, but I included them To make the tutorial use DirectX on Windows And
OpenGL on all other platforms like Linux. As you can see, there is also a
unusual call To IrrlichtDevice::setResizeAble(). This makes the render window
resizeable, which is quite useful For a mesh viewer.
EndRem

' ask user For driver

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device And Exit If creation failed

device = ..
	IrrlichtDevice.Create(driverType,_DIMENSION2DI(800,600),16,False,False,False, ..
	IEventReceiver.Create(MyEventReceiver.generate))

If Not device Return ' could Not create selected driver.

Device.setResizeAble(True)

Device.setWindowCaption("Irrlicht Engine - Loading...")

Local driver:IVideoDriver=device.getVideoDriver()
Local env:IGUIEnvironment=device.getGUIEnvironment()
Local smgr:ISceneManager=device.getSceneManager()
smgr.getParameters().setBoolAttributeByName(COLLADA_CREATE_SCENE_INSTANCES, True)

driver.setTextureCreationFlag(ETCF_ALWAYS_32_BIT, True)

smgr.addLightSceneNode()
smgr.addLightSceneNode(Null, _VECTOR3DF(50,-50,100), ..
	_SCOLORF(1.0,1.0,1.0),20000)
'  add our media directory as "search path"
Device.getFileSystem().addFolderFileArchive ( "../../media/" )

Rem
The Next Step is To read the configuration file. It is stored in the xml
format And looks a little bit like this:

@verbatim
<?xml version="1.0"?>
<config>
	<startUpModel file="some filename" />
	<messageText caption="Irrlicht Engine Mesh Viewer">
		Hello!
	</messageText>
</config>
@endverbatim

We need the data stored in there To be written into the Global variables
StartUpModelFile, MessageText And Caption. This is now done using the
Irrlicht Engine integrated XML parser:
EndRem

' read configuration from xml file

Local xml:IXMLReader = Device.getFileSystem().createXMLReader("config.xml")

While(xml And xml.read())

	Select xml.getNodeType()
		Case EXN_TEXT
			'  in this xml file, the only text which occurs is the messageText
			MessageText = xml.getNodeData()
		Case EXN_ELEMENT
			If (xml.getNodeName()="startUpModel")
				StartUpModelFile = xml.getAttributeValue("file")
			ElseIf (xml.getNodeName()="messageText")
				Caption = xml.getAttributeValue("caption")
			EndIf
	EndSelect
Wend

If (xml) Then xml.drop() '  don't forget to delete the xml reader 

If (AppArgs.length > 1) Then ..
	StartUpModelFile = AppArgs[1]

Rem
That wasn't difficult. Now we'll set a nicer font and create the Menu.
It is possible To Create submenus For every menu item. The call
menu.addItem("File", -1, True, True) For example adds a New menu
Item with the name "File" And the id -1. The following parameter says
that the menu item should be enabled, And the last one says, that there
should be a submenu. The submenu can now be accessed with
menu.getSubMenu(0), because the "File" entry is the menu item with
index 0.
EndRem

' set a nicer font

Local skin:IGUISkin = env.getSkin()
Local font:IGUIFont = env.getFont("fonthaettenschweiler.bmp")
If (font) Then ..
	skin.setFont(font)

' Create menu
Local menu:IGUIContextMenu = env.addMenu()
menu.addItem("File", -1, True, True)
menu.addItem("View", -1, True, True)
menu.addItem("Camera", -1, True, True)
menu.addItem("Help", -1, True, True)

Local submenu:IGUIContextMenu
submenu = menu.getSubMenu(0)
submenu.addItem("Open Model File & Texture...", 100)
submenu.addItem("Set Model Archive...", 101)
submenu.addItem("Load as Octree", 102)
submenu.addSeparator()
submenu.addItem("Quit", 200)

submenu = menu.getSubMenu(1)
submenu.addItem("sky box visible", 300, True, False, True)
submenu.addItem("toggle model debug information", 400, True, True)
submenu.addItem("model material", -1, True, True )

submenu = submenu.getSubMenu(1)
submenu.addItem("Off", 401)
submenu.addItem("Bounding Box", 410)
submenu.addItem("Normals", 420)
submenu.addItem("Skeleton", 430)
submenu.addItem("Wire overlay", 440)
submenu.addItem("Half-Transparent", 450)
submenu.addItem("Buffers bounding boxes", 460)
submenu.addItem("All", 499)

submenu = menu.getSubMenu(1).getSubMenu(2)
submenu.addItem("Solid", 610)
submenu.addItem("Transparent", 620)
submenu.addItem("Reflection", 630)

submenu = menu.getSubMenu(2)
submenu.addItem("Maya Style", 1000)
submenu.addItem("First Person", 1100)

submenu = menu.getSubMenu(3)
submenu.addItem("About", 500)

Rem
Below the menu we want a toolbar, onto which we can place colored
buttons And important looking stuff like a senseless combobox.
EndRem

' Create toolbar

Local bar:IGUIToolBar = env.addToolBar()

Local image:ITexture = driver.getTexture("open.png")

bar.addButton(1102, Null, "Open a mode",image, 0, False, True)

image = driver.getTexture("tools.png")
bar.addButton(1104, Null, "Open Toolset",image, 0, False, True)

image = driver.getTexture("zip.png")
bar.addButton(1105, Null, "Set Model Archive",image, 0, False, True)

image = driver.getTexture("help.png")
bar.addButton(1103, Null, "Open Help", image, 0, False, True)

' Create a combobox with some senseless texts

Local box:IGUIComboBox = env.addComboBox(_RECTI(250,4,350,23), bar, 108)

box.addItem("No filtering")
box.addItem("Bilinear")
box.addItem("Trilinear")
box.addItem("Anisotropic")
box.addItem("Isotropic")

Rem
To make the editor look a little bit better, we disable transparent gui
elements, And add an Irrlicht Engine logo. In addition, a text showing
the current frames per second value is created And the window caption is
changed.
EndRem

' disable alpha

For Local j:Int=0 Until EGDC_COUNT
	Local col:SColor = env.getSkin().GetColor(j)
	col.SetAlpha(255)
	env.getSkin().SetColor(j, col)
Next

' add a tabcontrol

createToolBox()

' Create fps text

Local fpstext:IGUIStaticText = ..
	env.addStaticText("", _RECTI(400,4,570,23), True, False, bar)

Local postext:IGUIStaticText = ..
	env.addStaticText("", _RECTI(10,50,470,80), False, False, Null, 2001)

postext.setVisible(False)

' set window caption

Caption :+ " - ["
Caption :+ driver.getName()
Caption :+ "]"
Device.setWindowCaption(Caption)

Rem
That's nearly the whole application. We simply show the about message
box at start up, And Load the first model. To make everything look
better, a skybox is created And a user controled camera, To make the
application a little bit more interactive. Finally, everything is drawn
in a standard drawing loop.
EndRem
DebugLog(StartUpModelFile)
' show about message box And Load Default model
If (AppArgs.length = 1) Then ..
	showAboutText()
loadModel(StartUpModelFile)

' add skybox
SkyBox = smgr.addSkyBoxSceneNode( ..
	driver.getTexture("irrlicht2_up.jpg"), ..
	driver.getTexture("irrlicht2_dn.jpg"), ..
	driver.getTexture("irrlicht2_lf.jpg"), ..
	driver.getTexture("irrlicht2_rt.jpg"), ..
	driver.getTexture("irrlicht2_ft.jpg"), ..
	driver.getTexture("irrlicht2_bk.jpg"))

' add a camera scene node
Cameras[0] = smgr.addCameraSceneNodeMaya()
Cameras[0].setFarValue(20000)
' Maya cameras reposition themselves relative To their target, so target the location
' where the mesh scene node is placed.
Cameras[0].setTarget(_VECTOR3DF(0,30,0))

Cameras[1] = smgr.addCameraSceneNodeFPS()
Cameras[1].setFarValue(20000)
Cameras[1].setPosition(_VECTOR3DF(0,0,-70))
Cameras[1].setTarget(_VECTOR3DF(0,30,0))

setActiveCamera(Cameras[0])

' Load the irrlicht engine logo
Local img:IGUIImage = ..
	env.addImageFromTexture(driver.getTexture("irrlichtlogo2.png"), ..
	_POSITION2DI(10, driver.getScreenSize().getHeight() - 128))

' lock the logo's edges to the bottom left corner of the screen
img.setAlignment(EGUIA_UPPERLEFT, EGUIA_UPPERLEFT, ..
		EGUIA_LOWERRIGHT, EGUIA_LOWERRIGHT)

' draw everything

While(Device.run() And driver)
	If (Device.isWindowActive())
		driver.beginScene(True, True, _SCOLOR(150,50,50,50))

		smgr.drawAll()
		env.drawAll()

		driver.endScene()

		Local str:String="FPS: "
		str :+ driver.getFPS()
		str :+ " Tris: "
		str :+ driver.getPrimitiveCountDrawn()
		fpstext.setText(str)

		Local cam:ICameraSceneNode = Device.getSceneManager().getActiveCamera()
		str = "Pos: "
		str :+ cam.getPosition().getX()
		str :+ " "
		str :+ cam.getPosition().getY()
		str :+ " "
		str :+ cam.getPosition().getZ()
		str :+ " Tgt: "
		str :+ cam.getTarget().getX()
		str :+ " "
		str :+ cam.getTarget().getY()
		str :+ " "
		str :+ cam.getTarget().getZ()
		postext.setText(str)
	Else
		Device.yield()
	EndIf
EndWhile

Device.drop()
