Rem
* Example 010 Shaders

This tutorial shows how To use shaders For D3D8, D3D9, And OpenGL with the
engine And how To Create New material types with them. It also shows how To
disable the generation of mipmaps at texture loading, And how To use text scene
nodes.

This tutorial does Not explain how shaders work. I would recommend To read the
D3D Or OpenGL documentation, To search a tutorial, Or To read a book about
this.

At first, we need To Include all headers And do the stuff we always do, like in
nearly all other tutorials:
EndRem
SuperStrict
Framework Irrlicht.Core

Rem
Because we want To use some interesting shaders in this tutorials, we need To
set some data For them To make them able To compute nice colors. In this
example, we'll use a simple vertex shader which will calculate the color of the
vertex based on the position of the camera.
For this, the shader needs the following data: The inverted world matrix For
transforming the normal, the clip matrix For transforming the position, the
camera position And the world position of the Object For the calculation of the
angle of light, And the color of the light. To be able To tell the shader all
this data every frame, we have To derive a class from the
IShaderConstantSetCallBack interface And override its only Method, namely
OnSetConstants(). This Method will be called every time the material is set.
The Method setVertexShaderConstant() of the IMaterialRendererServices interface
is used To set the data the shader needs. If the user chose To use a High Level
shader language like HLSL instead of Assembler in this example, you have To set
the variable name as parameter instead of the register index.
EndRem

Global device:IrrlichtDevice
Global UseHighLevelShaders:Int = False

Type MyShaderCallBack Extends IShaderConstantSetCallBack

	Method OnSetConstants(services:IMaterialRendererServices,userdata:Int)
		Local driver:IVideoDriver = services.getVideoDriver()

		' set inverted world matrix
		' If we are using highlevel shaders (the user can Select this when
		' starting the program), we must set the constants by name.

		Local invWorld:Matrix4 = driver.getTransform(ETS_WORLD)
		invWorld.makeInverse()

		If (UseHighLevelShaders)
			services.setVertexShaderConstantFromName("mInvWorld", invWorld.pointer(),16)
		Else
			services.setVertexShaderConstant(invWorld.pointer(), 0, 4)
		EndIf
		
		' set clip matrix

		Local worldViewProj:Matrix4
		worldViewProj = driver.getTransform(ETS_PROJECTION)
		worldViewProj.MultEq(driver.getTransform(ETS_VIEW))
		worldViewProj.MultEq(driver.getTransform(ETS_WORLD))

		If (UseHighLevelShaders)
			services.setVertexShaderConstantFromName("mWorldViewProj", worldViewProj.pointer(),16)
		Else
			services.setVertexShaderConstant(worldViewProj.pointer(), 4, 4)
		EndIf
		
		' set camera position

		Local pos:Vector3df = device.getSceneManager().getActiveCamera().getAbsolutePosition()

		If (UseHighLevelShaders)
			services.setVertexShaderConstantFromName("mLightPos",pos.getArray(),3)
		Else
			services.setVertexShaderConstant(pos.getArray(), 8, 1)
		EndIf

		' set light color

		Local col:SColorf=_SCOLORF(0.0,1.0,1.0,0.0)

		If (UseHighLevelShaders)
			services.setVertexShaderConstantFromName("mLightColor",col.getArray(),4)
		Else
			services.setVertexShaderConstant(col.getArray(), 9, 1)
		EndIf

		' set transposed world matrix
			
		Local world:Matrix4 = driver.getTransform(ETS_WORLD)
		world = world.getTransposed()

		If (UseHighLevelShaders)
			services.setVertexShaderConstantFromName("mTransWorld",world.pointer(),16)
		Else
			services.setVertexShaderConstant(world.pointer(), 10, 4)
		EndIf

	EndMethod

	' we must override the generate function in order for instantiation to work properly
	Function generate:IShaderConstantSetCallBack()
		Return New MyShaderCallBack
	EndFunction
EndType

Rem
The Next few lines start up the engine just like in most other tutorials
before. But in addition, we ask the user If he wants To use high level shaders
in this example, If he selected a driver which is capable of doing so.
EndRem

' let user Select driver Type

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

'  ask the user If we should use high level shaders For this example
If (driverType = EDT_DIRECT3D9 Or driverType = EDT_OPENGL)

	i=Input("Please press 'y' if you want to use high level shaders."+ ..
		Chr(13)+Chr(10)+Chr(13)+Chr(10))
	If (i = "y") Then UseHighLevelShaders = True
EndIf

' create device And Exit If creation failed

device = IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480))

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()
Local gui:IGUIEnvironment = device.getGUIEnvironment()

Rem
Now For the more interesting parts. If we are using Direct3D, we want
To Load vertex And pixel shader programs, If we have OpenGL, we want To
use ARB fragment And vertex programs. I wrote the corresponding
programs down into the files d3d8.ps, d3d8.vs, d3d9.ps, d3d9.vs,
opengl.ps And opengl.vs. We only need the Right filenames now. This is
done in the following switch. Note, that it is Not necessary To write
the shaders into text files, like in this example. You can even write
the shaders directly as strings into the cpp source file, And use later
addShaderMaterial() instead of addShaderMaterialFromFiles().
EndRem

Local vsFileName:String="" ' filename For the vertex shader
Local psFileName:String="" ' filename For the pixel shader	

Select driverType
	Case EDT_DIRECT3D8
		psFileName = "../../media/d3d8.psh"
		vsFileName = "../../media/d3d8.vsh"

	Case EDT_DIRECT3D9
		If (UseHighLevelShaders)
			psFileName = "../../media/d3d9.hlsl"
			vsFileName = psFileName ' both shaders are in the same file
		Else
			psFileName = "../../media/d3d9.psh"
			vsFileName = "../../media/d3d9.vsh"
		EndIf		
		
	Case EDT_OPENGL
		If (UseHighLevelShaders)
			psFileName = "../../media/opengl.frag"
			vsFileName = "../../media/opengl.vert"
		Else
			psFileName = "../../media/opengl.psh"
			vsFileName = "../../media/opengl.vsh"
		EndIf
EndSelect

Rem
In addition, we check If the hardware And the selected renderer is
capable of executing the shaders we want. If Not, we simply set the
filename String To 0. This is Not necessary, but useful in this
example: For example, If the hardware is able To execute vertex shaders
but Not pixel shaders, we Create a New material which only uses the
vertex shader, And no pixel shader. Otherwise, If we would tell the
engine To Create this material And the engine sees that the hardware
wouldn't be able to fullfill the request completely, it would not
Create any New material at all. So in this example you would see at
least the vertex shader in action, without the pixel shader.
EndRem

If (Not driver.queryFeature(EVDF_PIXEL_SHADER_1_1) And ..
	Not driver.queryFeature(EVDF_ARB_FRAGMENT_PROGRAM_1))

	device.getLogger().LogC("WARNING: Pixel shaders disabled "+ ..
		"because of missing driver/hardware support.")
		
	psFileName = ""
EndIf

If (Not driver.queryFeature(EVDF_VERTEX_SHADER_1_1) And ..
	Not driver.queryFeature(EVDF_ARB_VERTEX_PROGRAM_1))

	device.getLogger().LogC("WARNING: Vertex shaders disabled "+ ..
		"because of missing driver/hardware support.")
		
	vsFileName = ""
EndIf

Rem
Now lets Create the New materials. As you maybe know from previous
examples, a material Type in the Irrlicht engine is set by simply
changing the MaterialType value in the SMaterial struct. And this value
is just a simple 32 bit value, like video::EMT_SOLID. So we only need
the engine To Create a New value For us which we can set there. To do
this, we get a pointer To the IGPUProgrammingServices And call
addShaderMaterialFromFiles(), which returns such a New 32 bit value.
That's all.

The parameters To this Method are the following: First, the names of
the files containing the code of the vertex And the pixel shader. If
you would use addShaderMaterial() instead, you would Not need file
names, Then you could write the code of the shader directly as String.
The following parameter is a pointer To the IShaderConstantSetCallBack
class we wrote at the beginning of this tutorial. If you don't want to
set constants, set this To 0. The last paramter tells the engine which
material it should use as base material.

To demonstrate this, we Create two materials with a different base
material, one with EMT_SOLID And one with EMT_TRANSPARENT_ADD_COLOR.
EndRem

' Create materials

Local gpu:IGPUProgrammingServices = driver.getGPUProgrammingServices()
Local newMaterialType1:Int = 0
Local newMaterialType2:Int = 0

If (gpu)
	
	Local mc:IShaderConstantSetCallBack = ..
		IShaderConstantSetCallBack.Create(MyShaderCallback.generate)

	' Create the shaders depending on If the user wanted high level
	' Or low level shaders:

	If (UseHighLevelShaders)

		' create material from high level shaders (hlsl Or glsl)

		newMaterialType1 = gpu.addHighLevelShaderMaterialFromFiles( ..
			vsFileName, "vertexMain", EVST_VS_1_1, ..
			psFileName, "pixelMain", EPST_PS_1_1, ..
			mc, EMT_SOLID)

		newMaterialType2 = gpu.addHighLevelShaderMaterialFromFiles( ..
			vsFileName, "vertexMain", EVST_VS_1_1, ..
			psFileName, "pixelMain", EPST_PS_1_1, ..
			mc, EMT_TRANSPARENT_ADD_COLOR)				
	Else
		' create material from low level shaders (asm Or arb_asm)

		newMaterialType1 = gpu.addShaderMaterialFromFiles(vsFileName, ..
			psFileName, mc, EMT_SOLID)

		newMaterialType2 = gpu.addShaderMaterialFromFiles(vsFileName, ..
			psFileName, mc, EMT_TRANSPARENT_ADD_COLOR)
	EndIf
	
	mc.drop()
EndIf

Rem
Now it's time for testing the materials. We create a test cube and set
the material we created. In addition, we add a text scene node To the
cube And a rotation animator To make it look more interesting And
important.
EndRem

' Create test scene node 1, with the New created material Type 1

Local node:ISceneNode = smgr.addCubeSceneNode(50)
node.setPosition(_VECTOR3DF(0,0,0))
node.setMaterialTexture(0, driver.getTexture("../../media/wall.bmp"))
node.setMaterialFlag(EMF_LIGHTING, False)
node.setMaterialType(newMaterialType1)

smgr.addTextSceneNode(gui.getBuiltInFont(), ..
		"PS & VS & EMT_SOLID", ..
		_SCOLOR(255,255,255,255), node)

Local anim:ISceneNodeAnimator = smgr.createRotationAnimator( ..
	_VECTOR3DF(0,0.3,0))
node.addAnimator(anim)
anim.drop()

Rem
Same For the second cube, but with the second material we created.
EndRem

' Create test scene node 2, with the New created material Type 2

node = smgr.addCubeSceneNode(50)
node.setPosition(_VECTOR3DF(0,-10,50))
node.setMaterialTexture(0, driver.getTexture("../../media/wall.bmp"))
node.setMaterialFlag(EMF_LIGHTING, False)
node.setMaterialType(newMaterialType2)

smgr.addTextSceneNode(gui.getBuiltInFont(), ..
		"PS & VS & EMT_TRANSPARENT", ..
		_SCOLOR(255,255,255,255), node)

anim = smgr.createRotationAnimator(_VECTOR3DF(0,0.3,0))
node.addAnimator(anim)
anim.drop()

Rem
Then we add a third cube without a shader on it, To be able To compare
the cubes.
EndRem

' add a scene node with no shader

node = smgr.addCubeSceneNode(50)
node.setPosition(_VECTOR3DF(0,50,25))
node.setMaterialTexture(0, driver.getTexture("../../media/wall.bmp"))
node.setMaterialFlag(EMF_LIGHTING, False)
smgr.addTextSceneNode(gui.getBuiltInFont(), "NO SHADER", ..
	_SCOLOR(255,255,255,255), node)
	
Rem
And last, we add a skybox And a user controlled camera To the scene.
For the skybox textures, we disable mipmap generation, because we don't
need mipmaps on it.
EndRem

' add a nice skybox

driver.setTextureCreationFlag(ETCF_CREATE_MIP_MAPS, False)

smgr.addSkyBoxSceneNode( ..
	driver.getTexture("../../media/irrlicht2_up.jpg"), ..
	driver.getTexture("../../media/irrlicht2_dn.jpg"), ..
	driver.getTexture("../../media/irrlicht2_lf.jpg"), ..
	driver.getTexture("../../media/irrlicht2_rt.jpg"), ..
	driver.getTexture("../../media/irrlicht2_ft.jpg"), ..
	driver.getTexture("../../media/irrlicht2_bk.jpg"))
	
driver.setTextureCreationFlag(ETCF_CREATE_MIP_MAPS, True)

' add a camera And disable the mouse cursor

Local cam:ICameraSceneNode = smgr.addCameraSceneNodeFPS(Null, 100.0, .1)
cam.setPosition(_VECTOR3DF(-100,50,100))
cam.setTarget(_VECTOR3DF(0,0,0))
device.getCursorControl().setVisible(False)

Rem
Now draw everything. That's all.
EndRem

Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())

		driver.beginScene(True, True, _SCOLOR(255,0,0,0))
		smgr.drawAll()
		driver.endScene()

		Local fps:Int = driver.getFPS()

		If (lastFPS <> fps)
			Local str:String = "Irrlicht Engine - Vertex and pixel shader example ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

device.drop()
	
Rem
Compile And run this, And I hope you have fun with your New little shader
writing tool :).
EndRem
