Rem
* Example 015 Loading Scenes from .irr Files

Since version 1.1, Irrlicht is able To save And Load
the full scene graph into an .irr file, an xml based
format. There is an editor available To edit
those files, named irrEdit (http:'www.ambiera.com/irredit)
which can also be used as world And particle editor.
This tutorial shows how To use .irr files.

Lets start: Create an Irrlicht device And setup the window.
EndRem

SuperStrict
Framework Irrlicht.Core

' ask user For driver

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' create device And Exit If creation failed

Local device:IrrlichtDevice = IrrlichtDevice.Create(driverType, _DIMENSION2DI(640, 480))

If Not device Return ' could Not create selected driver.

device.setWindowCaption("Load .irr file example")

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

Rem
Now Load our .irr file.
.irr files can store the whole scene graph including animators,
materials And particle systems. And there is also the possibility To
store arbitrary user data For every scene node in that file. To keep
this example simple, we are simply loading the scene here. See the
documentation at ISceneManager::loadScene And ISceneManager::saveScene
For more information. So To Load And display a complicated huge scene,
we only need a single call To loadScene().
EndRem

' Load the scene
smgr.loadScene("../../media/example.irr")

Rem
Now we'll create a camera, and give it a collision response animator
that's built from the mesh nodes in the scene we just loaded.
EndRem
Local camera:ICameraSceneNode = smgr.addCameraSceneNodeFPS(Null, 50, 0.1)

' Create a meta triangle selector To hold several triangle selectors.
Local meta:IMetaTriangleSelector = smgr.createMetaTriangleSelector()

Rem
Now we will find all the nodes in the scene And Create triangle
selectors For all suitable nodes.  Typically, you would want To make a
more informed decision about which nodes To performs collision checks
on you could capture that information in the node name Or Id.
EndRem
Local nodes:ISceneNode[] = ..
	smgr.getSceneNodesFromType(ESNT_ANY) ' Find all nodes

For Local i:Int = 0 Until nodes.Dimensions()[0]

	Local node:ISceneNode = nodes[i]
	Local selector:ITriangleSelector = Null

	Select node.getType()
		Case ESNT_CUBE 
			selector = smgr.createTriangleSelectorFromBoundingBox(node)

		Case ESNT_ANIMATED_MESH
			' Because the selector won't animate with the mesh,
			' And is only being used For camera collision, we'll just use an approximate
			' bounding box instead of ((scene::IAnimatedMeshSceneNode*)node).getMesh(0)
			selector = smgr.createTriangleSelectorFromBoundingBox(node)
	
		Case ESNT_MESH 
			selector = smgr.createTriangleSelector(IMeshSceneNode(node).getMesh(), node)

		Case ESNT_SPHERE ' Derived from IMeshSceneNode
			selector = smgr.createTriangleSelector(IMeshSceneNode(node).getMesh(), node)
	
		Case ESNT_TERRAIN
			selector = smgr.createTerrainTriangleSelector(ITerrainSceneNode(node))
	
		Case ESNT_OCT_TREE
			selector = smgr.createOctTreeTriangleSelector(IMeshSceneNode(node).getMesh(), node)
	EndSelect
	
	If (selector)
		' Add it To the meta selector, which will take a reference To it
		meta.addTriangleSelector(selector)
		' And drop my reference To it, so that the meta selector owns it.
		selector.drop()
	EndIf
Next

Rem
Now that the mesh scene nodes have had triangle selectors created And added
To the meta selector, Create a collision response animator from that meta selector.
EndRem
Local anim:ISceneNodeAnimator = smgr.createCollisionResponseAnimator( ..
	meta, camera, _VECTOR3DF(5,5,5), ..
	_VECTOR3DF(0,0,0))
meta.drop() ' I'm done with the meta selector now

camera.addAnimator(anim)
anim.drop() ' I'm done with the animator now

' And set the camera position so that it doesn't start off stuck in the geometry
camera.setPosition(_VECTOR3DF(0, 20, 0))

' Point the camera at the cube node, by finding the first node of Type ESNT_CUBE
Local cube:ISceneNode = smgr.getSceneNodeFromType(ESNT_CUBE)
If(cube) Then ..
	camera.setTarget(cube.getAbsolutePosition())

Rem
That's it. Draw everything and finish as usual.
EndRem

Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())
		driver.beginScene(True, True, _SCOLOR(0,200,200,200))
		smgr.drawAll()
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
	
		If (lastFPS <> fps)
			Local str:String = "Load Irrlicht File example - Irrlicht Engine ["
			str :+ driver.getName()
		 	str :+ "] FPS:"
		  	str :+ fps
	
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

device.drop()

