Rem
* Example 016 Quake3 Map Shader Support

This Tutorial shows how To Load a Quake 3 map into the
engine, Create a SceneNode For optimizing the speed of
rendering And how To Create a user controlled camera.

Lets start like the HelloWorld example: We Include
the irrlicht header files And an additional file To be able
To ask the user For a driver Type using the console.
EndRem
SuperStrict
Framework Irrlicht.Core
Import BRL.Random

Rem
	define which Quake3 Level should be loaded
EndRem
'Const QUAKE3_STORAGE_FORMAT	addZipFileArchive
Const QUAKE3_STORAGE_1:String = "../../media/map-20kdm2.pk3"
Const QUAKE3_MAP_NAME:String = 	"maps/20kdm2.bsp"

Rem
As already written in the HelloWorld example, in the Irrlicht
Engine, everything can be found in the namespace 'irr'.
To get rid of the irr:: in front of the name of every class,
we tell the compiler that we use that namespace from now on,
And we will Not have To write that 'irr::'.
There are 5 other sub namespaces 'core', 'scene', 'video',
'io' and 'gui'. Unlike in the HelloWorld example,
we do Not a 'using namespace' for these 5 other namespaces
because in this way you will see what can be found in which
namespace. But If you like, you can also Include the namespaces
like in the previous example. Code just like you want To.
EndRem
using namespace irr
using namespace scene

Rem
Again, To be able To use the Irrlicht.DLL file, we need To link with the
Irrlicht.lib. We could set this option in the project settings, but
To make it easy, we use a pragma comment lib:
EndRem
#ifdef _MSC_VER
#pragma comment(lib, "Irrlicht.lib")
#EndIf


//! produces a serie of screenshots
class CScreenShotFactory : Public IEventReceiver
{
Public:

	CScreenShotFactory( IrrlichtDevice *device, Const c8 * templateName, ISceneNode* node )
		: Device(device), Number(0), FilenameTemplate(templateName), Node(node)
	{
		FilenameTemplate.Replace ( '/', '_' )
		FilenameTemplate.Replace ( '\\', '_' )
	}

	bool OnEvent(Const SEvent& event)
	{
		// check If user presses the key F9
		If ((event.EventType == EET_KEY_INPUT_EVENT) &&
				event.KeyInput.PressedDown)
		{
			If (event.KeyInput.Key == KEY_F9)
			{
				video::IImage* image = Device.getVideoDriver().createScreenShot()
				If (image)
				{
					c8 buf[256]
					snprintf(buf, 256, "%s_shot%04d.jpg",
							FilenameTemplate.c_str(),
							++Number)
					Device.getVideoDriver().writeImageToFile(image, buf, 85 )
					image.drop()
				}
			}
			Else
			If (event.KeyInput.Key == KEY_F8)
			{
				Node.setDebugDataVisible(scene::EDS_BBOX_ALL)
			}
		}
		Return False
	}

Private:
	IrrlichtDevice *Device
	u32 Number
	core::stringc FilenameTemplate
	ISceneNode* Node
}


Rem
Ok, lets start.
EndRem

Int IRRCALLCONV main(Int argc, char* argv[])
{
	Rem
	Like in the HelloWorld example, we Create an IrrlichtDevice with
	createDevice(). The difference now is that we ask the user To Select
	which hardware accelerated driver To use. The Software device would be
	too slow To draw a huge Quake 3 map, but just For the fun of it, we make
	this decision possible too.
	EndRem

	// ask user For driver

	video::E_DRIVER_TYPE driverType

	printf("Please select the driver you want for this example:\n"\
		" (a) Direct3D 9.0c\n (b) Direct3D 8.1\n (c) OpenGL 1.5\n"\
		" (d) Software Renderer\n (e) Burning's Software Renderer\n"\
		" (f) NullDevice\n (otherKey) exit\n\n")

	char i
	std::cin >> i

	switch(i)
	{
		Case 'a': driverType = video::EDT_DIRECT3D9break
		Case 'b': driverType = video::EDT_DIRECT3D8break
		Case 'c': driverType = video::EDT_OPENGL   break
		Case 'd': driverType = video::EDT_SOFTWARE break
		Case 'e': driverType = video::EDT_BURNINGSVIDEObreak
		Case 'f': driverType = video::EDT_NULL     break
		Default: Return 1
	}	

	// Create device And Exit If creation failed
	Const core::dimension2di videoDim ( 800,600 )

	IrrlichtDevice *device = createDevice(driverType, videoDim, 32, False )

	If (device == 0)
		Return 1 // could Not Create selected driver.

	Const char* mapname=0
	If (argc>2)
		mapname = argv[2]
	Else
		mapname = QUAKE3_MAP_NAME

	Rem
	Get a pointer To the video driver And the SceneManager so that
	we do Not always have To write device.getVideoDriver() And
	device.getSceneManager().
	EndRem
	video::IVideoDriver* driver = device.getVideoDriver()
	scene::ISceneManager* smgr = device.getSceneManager()
	gui::IGUIEnvironment* gui = device.getGUIEnvironment()

	//! add our Private media directory To the file system
	device.getFileSystem().addFolderFileArchive("../../media/")

	Rem
	To display the Quake 3 map, we first need To Load it. Quake 3 maps
	are packed into .pk3 files, which are nothing other than .zip files.
	So we add the .pk3 file To our FileSystem. After it was added,
	we are able To read from the files in that archive as they would
	directly be stored on disk.
	EndRem
	If (argc>2)
		device.getFileSystem().QUAKE3_STORAGE_FORMAT (argv[1])
	Else
		device.getFileSystem().QUAKE3_STORAGE_FORMAT ( QUAKE3_STORAGE_1 )
#ifdef QUAKE3_STORAGE_2
	device.getFileSystem().QUAKE3_STORAGE_FORMAT ( QUAKE3_STORAGE_2 )
#EndIf



	Rem
	Now we can Load the mesh by calling getMesh(). We get a pointer returned
	To a IAnimatedMesh. As you know, Quake 3 maps are Not really animated,
	they are only a huge chunk of static geometry with some materials
	attached. Hence the IAnimated mesh consists of only one frame,
	so we get the "first frame" of the "animation", which is our quake level
	And Create an OctTree scene node with it, using addOctTreeSceneNode().
	The OctTree optimizes the scene a little bit, trying To draw only geometry
	which is currently visible. An alternative To the OctTree would be a
	AnimatedMeshSceneNode, which would draw always the complete geometry of
	the mesh, without optimization. Try it out: Write addAnimatedMeshSceneNode
	instead of addOctTreeSceneNode And compare the primitives drawed by the
	video driver. (There is a getPrimitiveCountDrawed() Method in the
	IVideoDriver class). Note that this optimization with the Octree is only
	useful when drawing huge meshes consisting of lots of geometry.
	EndRem
	scene::IQ3LevelMesh* mesh =
		(scene::IQ3LevelMesh*) smgr.getMesh(mapname)

	Rem
		add the geometry mesh To the Scene ( polygon & patches )
		The Geometry mesh is optimised For faster drawing
	EndRem
	scene::ISceneNode* node = 0
	If ( mesh )
	{
		scene::IMesh *geometry = mesh.getMesh(quake3::E_Q3_MESH_GEOMETRY)
//		node = smgr.addMeshSceneNode ( geometry )
		node = smgr.addOctTreeSceneNode(geometry, 0, -1, 1024)
	}

	// Create an event receiver For making screenshots
	CScreenShotFactory screenshotFactory ( device, mapname, node )
	device.setEventReceiver ( &screenshotFactory )

	Rem
		now construct SceneNodes For each Shader
		The Objects are stored in the quake mesh scene::E_Q3_MESH_ITEMS
		And the Shader ID is stored in the MaterialParameters
		mostly dark looking skulls And moving lava.. Or green flashing tubes?
	EndRem
	If ( mesh )
	{
		// the additional mesh can be quite huge And is unoptimized
		scene::IMesh * additional_mesh = mesh.getMesh ( quake3::E_Q3_MESH_ITEMS )

#ifdef SHOW_SHADER_NAME
		gui::IGUIFont *font = device.getGUIEnvironment().getFont("../../media/fontlucida.png")
		u32 count = 0
#EndIf

		For ( u32 i = 0 i!= additional_mesh.getMeshBufferCount () ++i )
		{
			IMeshBuffer *meshBuffer = additional_mesh.getMeshBuffer ( i )
			Const video::SMaterial &material = meshBuffer.getMaterial()

			//! The ShaderIndex is stored in the material parameter
			s32 shaderIndex = (s32) material.MaterialTypeParam2

			// the meshbuffer can be rendered without additional support, Or it has no shader
			Const quake3::SShader *shader = mesh.getShader ( shaderIndex )
			If ( 0 == shader )
			{
				Continue
			}

			// we can dump the shader To the console in its
			// original but already parsed layout in a pretty
			// printers way.. commented out, because the console
			// would be full...
			// quake3::dumpShader ( Shader )

#ifndef SHOW_SHADER_NAME
			smgr.addQuake3SceneNode ( meshBuffer, shader )
#Else
			// Now add the MeshBuffer(s) with the current Shader To the Manager
#If 0
			If (	shader.name != "textures/cf/window-decal"
				)
				Continue
#EndIf
			If ( 0 == count )
			{
				core::stringc s
				//quake3::dumpShader ( s, shader )
				printf ( s.c_str () )
			}
			count += 1

			node = smgr.addQuake3SceneNode ( meshBuffer, shader )

			core::stringw name( node.getName() )
			node = smgr.addBillboardTextSceneNode(
					font,
					name.c_str(),
					node,
					core::dimension2d<f32>(80.0f, 8.0f),
					_VECTOR3DF(0, 10, 0)
					)
#EndIf
		}


		// original mesh is Not needed anymore
		mesh.releaseMesh ( quake3::E_Q3_MESH_ITEMS )
	}

	Rem
	Now we only need a Camera To look at the Quake 3 map. And we want To
	Create a user controlled camera. There are some different cameras
	available in the Irrlicht engine. For example the Maya Camera which can
	be controlled compareable To the camera in Maya: Rotate with Left mouse
	button pressed, Zoom with both buttons pressed, translate with Right
	mouse button pressed. This could be created with
	addCameraSceneNodeMaya(). But For this example, we want To Create a
	camera which behaves like the ones in first person shooter games (FPS).
	EndRem

	scene::ICameraSceneNode* camera = smgr.addCameraSceneNodeFPS()

	Rem
		so we need a good starting Position in the level.
		we can ask the Quake3 Loader For all entities with class_name
		"info_player_deathmatch"
		we choose a random launch
		
	EndRem
	If ( mesh )
	{
		Const quake3::tQ3EntityList &entityList = mesh.getEntityList ()

		quake3::SEntity search
		search.name = "info_player_deathmatch"

		s32 index = entityList.binary_search_const ( search )
		If ( index >= 0 )
		{
			Const quake3::SVarGroup *group
			s32 notEndList
			do
			{
				group = entityList[ index ].getGroup(1)

				u32 parsepos = 0
				_VECTOR3DF pos =
					quake3::getAsVector3df ( group.get ( "origin" ), parsepos )

				parsepos = 0
				f32 angle = quake3::getAsFloat ( group.get ( "angle"), parsepos )

				_VECTOR3DF target ( 0.f, 0.f, 1.f )
				target.rotateXZBy ( angle, _VECTOR3DF () )

				camera.setPosition ( pos )
				camera.setTarget ( pos + target )

				index += 1
Rem
				notEndList = (	index < (s32) entityList.size () &&
								entityList[index].name == search.name &&
								(device.getTimer().getRealTime() >> 3 ) & 1
							)
EndRem
				notEndList = index == 2
			} While ( notEndList )
		}

	}

	Rem
	The mouse cursor needs Not To be visible, so we make it invisible.
	EndRem

	device.getCursorControl().setVisible(False)

	// Load the engine logo
	gui.addImage(driver.getTexture("irrlichtlogo2.png"),
			core::position2d<s32>(10, 10))

	// show the driver logo
	core::position2di pos ( videoDim.Width - 128, videoDim.Height - 64 )

	switch ( driverType )
	{
		Case video::EDT_BURNINGSVIDEO:
			gui.addImage(driver.getTexture("burninglogo.png"),pos)
			break
		Case video::EDT_OPENGL:
			gui.addImage(driver.getTexture("opengllogo.png"),pos)
			break
		Case video::EDT_DIRECT3D8:
		Case video::EDT_DIRECT3D9:
			gui.addImage(driver.getTexture("directxlogo.png"),pos)
			break
	}

	Rem
	We have done everything, so lets draw it. We also write the current
	frames per second And the drawn primitives To the caption of the
	window. The 'if (device.isWindowActive())' line is optional, but
	prevents the engine render To set the position of the mouse cursor
	after task switching when other program are active.
	EndRem
	Int lastFPS = -1

	While(device.run())
	If (device.isWindowActive())
	{
		driver.beginScene(True, True, video::SColor(255,20,20,40))
		smgr.drawAll()
		gui.drawAll()

		driver.endScene()

		Int fps = driver.getFPS()

		//If (lastFPS != fps)
		{
			io::IAttributes * attr = smgr.getParameters()

			s32 calls = attr.getAttributeAsInt ( "calls" )
			s32 culled = attr.getAttributeAsInt ( "culled" )

			core::stringw str = L"Q3 ["
			str += driver.getName()
			str += "] FPS:"
			str += fps
			str += " Cull:"
			str += calls
			str += "/"
			str += culled

			device.setWindowCaption(str.c_str())
			lastFPS = fps
		}
	}

	Rem
	In the End, Delete the Irrlicht device.
	EndRem
	device.drop()
	
	Return 0
}

Rem
*EndRem
