Rem
This Tutorial shows how To load a Quake 3 map into the
engine, create a SceneNode For optimizing the speed of
rendering And how To create a user controlled camera.

Lets start like the HelloWorld example: We Include
the irrlicht header files And an additional file To be able
To ask the user For a driver Type using the console.
EndRem
SuperStrict
Framework Irrlicht.Core
Import BRL.Random

' ! produces a serie of screenshots
Type CScreenShotFactory Extends IEventReceiver
	
	Method Init:CScreenShotFactory(device:IrrlichtDevice,templateName:String)
		'  store pointer To device so we can use it
		Self.Device=device
		'  start with zero
		Self.Number = 0
		Self.Filename=""
		Self.FilenameTemplate=templateName
		Return Self
	EndMethod
	
	Method OnEvent:Int(event:SEvent)

		'  check If user presses the key F9
		If (event.getEventType()=EET_KEY_INPUT_EVENT And ..
			event.getKeyInputKey()=EKEY_F9 And ..
			Not event.getKeyPressedDown())

			Local image:IImage = Device.getVideoDriver().createScreenShot()
			If (image)
				Filename=FilenameTemplate+"_shot"+Number+".jpg"
				Device.getVideoDriver().writeImageToFile(image, Filename, 85 )
				image.drop()
			EndIf
		EndIf
		Return False
	EndMethod

	Field Device:IrrlichtDevice
	Field Number:Int
	Field Filename:String
	Field FilenameTemplate:String
	
	' we must override the generate function in order for instantiation to work properly
	Function generate:IEventReceiver()
		Return New CScreenShotFactory 
	EndFunction
EndType

Rem
Ok, lets start. 
EndRem

Rem
Like in the HelloWorld example, we create an IrrlichtDevice with
createDevice(). The difference now is that we ask the user To Select 
which hardware accelerated driver To use. The Software device would be
too slow To draw a huge Quake 3 map, but just For the fun of it, we make
this decision possible too.
EndRem

'  ask user For driver

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' create device And Exit If creation failed
Local videoDim:Dimension2di=_DIMENSION2DI( 800,600 )

Local device:IrrlichtDevice = IrrlichtDevice.create(driverType, videoDim, 32, False )

If Not device Return ' could Not create selected driver.

Rem
Get a pointer To the video driver And the SceneManager so that
we do Not always have To write device.getVideoDriver() And
device.getSceneManager().
EndRem
Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

' ! add our Private media directory To the file system
device.getFileSystem().addFolderFileArchive("../../media/")

Rem
To display the Quake 3 map, we first need To load it. Quake 3 maps
are packed into .pk3 files wich are nothing other than .zip files.
So we add the .pk3 file To our FileSystem. After it was added,
we are able To read from the files in that archive as they would
directly be stored on disk.
EndRem
device.getFileSystem().addZipFileArchive("../../media/map-20kdm2.pk3")
	
Rem 
Now we can load the mesh by calling getMesh(). We get a pointer returned
To a IAnimatedMesh. As you know, Quake 3 maps are Not really animated,
they are only a huge chunk of static geometry with some materials
attached. Hence the IAnimated mesh consists of only one frame,
so we get the "first frame" of the "animation", which is our quake level
And create an OctTree scene node with it, using addOctTreeSceneNode().
The OctTree optimizes the scene a little bit, trying To draw only geometry
which is currently visible. An alternative To the OctTree would be a 
AnimatedMeshSceneNode, which would draw always the complete geometry of 
the mesh, without optimization. Try it out: Write addAnimatedMeshSceneNode
instead of addOctTreeSceneNode And compare the primitives drawed by the
video driver. (There is a getPrimitiveCountDrawed() Method in the 
IVideoDriver class). Note that this optimization with the Octree is only
useful when drawing huge meshes consisting of lots of geometry.
EndRem
Local tempMesh:IAnimatedMesh = smgr.getMesh("maps/20kdm2.bsp")
Local mesh:IQ3LevelMesh = IQ3LevelMesh.createFromHandle(tempMesh.handle,False)
'  create an event receiver For making screenshots
Local screenshotFactory:IEventReceiver = ..
	CScreenShotFactory(IEventReceiver.create(CScreenShotFactory.generate)).Init( device, "20kdm2" )
device.setEventReceiver ( screenshotFactory )

Rem
	add the geometry mesh To the Scene ( polygon & patches )
	The Geometry mesh is optimised For faster drawing
EndRem
Local node:ISceneNode
If ( mesh )
	Local geometry:IMesh = mesh.getMesh(E_Q3_MESH_GEOMETRY )
	node = smgr.addMeshSceneNode ( geometry )
EndIf

Rem
	now construct SceneNodes For each Shader
	The Objects are stored in the quake mesh scene::E_Q3_MESH_ITEMS
	And the Shader ID is stored in the MaterialParameters
	mostly dark looking skulls And moving lava.. Or green flashing tubes?
EndRem

' NOTE: something here is causing crash.  maybe improperly releasing?
If ( mesh )
	'  the additional mesh can be quite huge And is unoptimized
	Local additional_mesh:IMesh = mesh.getMesh ( E_Q3_MESH_ITEMS )

	For Local i:Int = 0 Until additional_mesh.getMeshBufferCount()

		Local meshBuffer:IMeshBuffer = additional_mesh.getMeshBuffer ( i )
		Local material:SMaterial = meshBuffer.getMaterial()

		' ! The ShaderIndex is stored in the material parameter
		Local shaderIndex:Int = material.getMaterialTypeParam2()

		'  the meshbuffer can be rendered without additional support, Or it has no shader
		Local shader:SShader = mesh.getShaderByIdx( shaderIndex )
		If ( Not shader )		
			Continue
		EndIf

		'  we can dump the shader To the console in it's original but already parsed layout
		'  in a pretty printers way.. commented out, because the console would be full...
		'  quake3::dumpShader ( Shader )

		'  Now add the MeshBuffer(s) with the current Shader To the Manager
		smgr.addQuake3SceneNode ( meshBuffer, shader )
	Next
	
	'  original mesh is Not needed anymore
	mesh.releaseMesh ( E_Q3_MESH_ITEMS )
EndIf

Rem
	Now we only need a Camera To look at the Quake 3 map.
	And we want To create a user controlled camera. There are some
	different cameras available in the Irrlicht engine. For example the 
	Maya Camera which can be controlled compareable To the camera in Maya:
	Rotate with Left mouse button pressed, Zoom with both buttons pressed,
	translate with Right mouse button pressed. This could be created with
	addCameraSceneNodeMaya(). But For this example, we want To create a 
	camera which behaves like the ones in first person shooter games (FPS).
EndRem

Local camera:ICameraSceneNode = smgr.addCameraSceneNodeFPS()

Rem
	so we need a good starting Position in the level.
	we can ask the Quake3 Loader For all entities with class_name "info_player_deathmatch"
	we choose a random launch	
EndRem

If ( mesh )
	Local entityList:SShader[] = mesh.getEntityList()

	Local index:Int=0

	For Local entity:SShader=EachIn entityList
		If entity.getName()="info_player_deathmatch" Then Exit
		index:+1
	Next

	If ( index >= 0 )
		Local group:SVarGroup		
		Local notEndList:Int

		SeedRnd device.getTimer().getRealTime()
		
		Repeat			
			group = entityList[ index ].getGroup(1)

			Local parsepos:Int = 0
			Local pos:Vector3df = Quake3_getAsVector3df( group.get("origin"), parsepos )

			parsepos = 0
			Local angle:Float = Quake3_getAsFloat( group.get("angle"), parsepos )

			Local target:Vector3df=_VECTOR3DF( 0, 0, 1.0 )
			target.rotateXZBy ( angle, _VECTOR3DF() )

			camera.setPosition ( pos )
			camera.setTarget ( pos.Plus(target) )
			index :+ 1
			notEndList = (	index < entityList.Dimensions()[0] And ..
							entityList[index].getName() = "info_player_deathmatch" And ..
							((Rand(15) Mod 5 ) <> 0) ..
						)
		Until ( Not notEndList )
	EndIf
EndIf

Rem
The mouse cursor needs Not To be visible, so we make it invisible.
EndRem

device.getCursorControl().setVisible(False)

'  load the engine logo
Local env:IGUIEnvironment = device.getGUIEnvironment()
env.addImageFromTexture(driver.getTexture("irrlichtlogo2.png"),_POSITION2DI(10, 10))

'  show the driver logo
Local pos:Position2di = _POSITION2DI( videoDim.getWidth() - 128, videoDim.getHeight() - 64 )

Select driverType 
	Case EDT_BURNINGVIDEO
		env.addImageFromTexture(driver.getTexture("burninglogo.png"),pos )
	Case EDT_OPENGL
		env.addImageFromTexture(driver.getTexture("opengllogo.png"),pos )
	Case EDT_DIRECT3D8
	Case EDT_DIRECT3D9
		env.addImageFromTexture(driver.getTexture("directxlogo.png"),pos )
EndSelect

Rem
We have done everything, so lets draw it. We also write the current
frames per second And the drawn primitives To the caption of the
window. The 'if (device.isWindowActive())' line is optional, but 
prevents the engine render To set the position of the mouse cursor 
after task switching when other program are active.
EndRem
Local lastFPS:Int = -1

While(device.run())
	If (device.isWindowActive())
		driver.beginScene(True, True, _SCOLOR(255,20,20,40))
		smgr.drawAll()
		env.drawAll()
	
		driver.endScene()
	
		Local fps:Int = driver.getFPS()
	
		If (lastFPS <> fps)		
			Local attr:IAttributes = smgr.getParameters()
	
			Local calls:Int = attr.getAttributeAsIntByName( "calls" )
			Local culled:Int = attr.getAttributeAsIntByName( "culled" )
	
			Local str:String = "Q3 ["
			str :+ driver.getName()
			str :+ "] FPS:"
			str :+ fps
			str :+ " Cull:"
			str :+ calls
			str :+ "/"
			str :+ culled
	
			device.setWindowCaption(str)
			lastFPS = fps
		EndIf
	EndIf
Wend

Rem
In the End, Delete the Irrlicht device.
EndRem
device.drop()

