Rem* Example 018 Splitscreen

A tutorial by Max Winkel.

In this tutorial we'll learn how to use splitscreen (e.g. for racing-games)
with Irrlicht. We'll create a viewport divided
into 4 parts, wtih 3 fixed cameras And one user-controlled.

Ok, let's start with the headers (I think there's
nothing To say about it)
EndRem

SuperStrict
Framework Irrlicht.Core

Rem
Now we'll define the resolution in a constant for use in
initializing the device And setting up the viewport. In addition
we set up a Global variable saying splitscreen is active Or Not.
EndRem

'Resolution
Const ResX:Int = 800
Const ResY:Int = 600
Const fullScreen:Int = False

'Use SplitScreen?
Global SplitScreen:Int = True

Rem
Now we need four pointers To our cameras which are created later:
EndRem

'cameras
Global camera:ICameraSceneNode[4]

Rem
In our event-receiver we switch the SplitScreen-variable,
whenever the user press the S-key. All other events are sent
To the FPS camera.
EndRem

Type MyEventReceiver Extends IEventReceiver

	Method OnEvent:Int(event:SEvent)

	    'Key S enables/disables SplitScreen
		If (event.getEventType()=EET_KEY_INPUT_EVENT And ..
			event.getKeyPressedDown()=True And ..
			event.getKeyInputKey()=EKEY_KEY_S)
			
		      SplitScreen = (SplitScreen=False)
     		 Return True
		EndIf
		'Send all other events To camera4
    		If (camera[3]<>Null And camera[3].isValid())
     		camera[3].OnEvent(event)
		EndIf

		Return False
	EndMethod

	' we must override the generate function in order for instantiation to work properly
	' must return IEventReceiver
	Function generate:IEventReceiver()
		Return IEventReceiver(New MyEventReceiver)
	EndFunction
EndType

Rem
Ok, now the main-Function:
First, we initialize the device, get the SourceManager And
VideoDriver, Load an animated mesh from .md2 And a map from
.pk3. Because that's old stuff, I won't explain every step.
Just take care of the maps position.
EndRem

Local driverType:Int=EDT_NULL

Local i:String=Input("Please select the driver you want for this example:"+Chr(13)+Chr(10)+ ..
	" (a) Direct3D 9.0c"+Chr(13)+Chr(10)+" (b) Direct3D 8.1"+Chr(13)+Chr(10)+ ..
	" (c) OpenGL 1.5"+Chr(13)+Chr(10)+" (d) Software Renderer"+Chr(13)+Chr(10)+ ..
	" (e) Burning's Software Renderer"+Chr(13)+Chr(10)+" (f) NullDevice"+Chr(13)+Chr(10)+ ..
	" (otherKey) Exit"+Chr(13)+Chr(10)+Chr(13)+Chr(10))

Select i
	Case "a" driverType=EDT_DIRECT3D9
	Case "b" driverType=EDT_DIRECT3D8
	Case "c" driverType=EDT_OPENGL
	Case "d" driverType=EDT_SOFTWARE
	Case "e" driverType=EDT_BURNINGVIDEO
	Case "f" driverType=EDT_NULL
	Default Return 
EndSelect

' Create device

Local receiver:MyEventReceiver = ..
	MyEventReceiver(IEventReceiver.Create(MyEventReceiver.generate))

Local device:IrrlichtDevice = IrrlichtDevice.Create(driverType, _DIMENSION2DI(ResX, ResY), ..
	32, fullScreen, False, False, receiver)

If Not device Return ' could Not create selected driver.

Local driver:IVideoDriver = device.getVideoDriver()
Local smgr:ISceneManager = device.getSceneManager()

'Load model
Local model:IAnimatedMesh = smgr.getMesh("../../media/sydney.md2")
If Not model Then Return
Local model_node:IAnimatedMeshSceneNode = smgr.addAnimatedMeshSceneNode(model)

'Load texture
If (model_node)
	Local texture:ITexture = driver.getTexture("../../media/sydney.bmp")
	model_node.setMaterialTexture(0,texture)
	model_node.setMD2Animation(EMAT_RUN)
	'Disable lighting (we've got no light)
	model_node.setMaterialFlag(EMF_LIGHTING,False)
EndIf

'Load map
device.getFileSystem().addZipFileArchive("../../media/map-20kdm2.pk3")
Local map:IAnimatedMesh = smgr.getMesh("20kdm2.bsp")
If (map) 
	Local map_node:ISceneNode = smgr.addOctTreeSceneNode(map.getMesh(0))
	'Set position
	map_node.setPosition(_VECTOR3DF(-850,-220,-850))
EndIf

Rem
Now we Create our four cameras. One is looking at the model
from the front, one from the top And one from the side. In
addition there's a FPS-camera which can be controlled by the
user.
EndRem
' Create 3 fixed And one user-controlled cameras
'Front
camera[0] = smgr.addCameraSceneNode(Null, _VECTOR3DF(50,0,0), _VECTOR3DF(0,0,0))
'Top
camera[1] = smgr.addCameraSceneNode(Null, _VECTOR3DF(0,50,0), _VECTOR3DF(0,0,0))
'Left
camera[2] = smgr.addCameraSceneNode(Null, _VECTOR3DF(0,0,50), _VECTOR3DF(0,0,0))
'User-controlled
camera[3] = smgr.addCameraSceneNodeFPS()
' don't start at sydney's position
If (camera[3]) Then ..
	camera[3].setPosition(_VECTOR3DF(-50,0,-50))

Rem
Create a variable For counting the fps And hide the mouse:
EndRem
'Hide mouse
device.getCursorControl().setVisible(False)
'We want To count the fps
Local lastFPS:Int=-1
Local fps:Int=0
Local str:String=""

Rem
There wasn't much new stuff - till now!
Only by defining four cameras, the game won't be splitscreen.
To do this you need several steps:
  - Set the viewport To the whole screen
  - Begin a New scene (Clear screen)

  - The following 3 steps are repeated For every viewport in the splitscreen
    - Set the viewport To the area you wish
    - Activate the camera which should be "linked" with the viewport
    - Render all objects

  - If you have a GUI:
    - Set the viewport the whole screen
    - Display the GUI
  - End scene

Sounds a little complicated, but you'll see it isn't:
EndRem

While(device.run())
    'Set the viewpoint the the whole screen And begin scene
	driver.SetViewport(_RECTI(0,0,ResX,ResY))
	driver.beginScene(True, True, _SCOLOR(0,100,100,100))

	'If SplitScreen is used
	If (SplitScreen)
		'Activate camera1
		smgr.setActiveCamera(camera[0])
		'Set viewpoint To the first quarter (Left top)
		driver.SetViewport(_RECTI(0,0,ResX/2,ResY/2))
		'Draw scene
		smgr.drawAll()
		
		'Activate camera2
		smgr.setActiveCamera(camera[1])
		'Set viewpoint To the second quarter (Right top)
		driver.SetViewport(_RECTI(ResX/2,0,ResX,ResY/2))
		'Draw scene
		smgr.drawAll()
		
		'Activate camera3
		smgr.setActiveCamera(camera[2])
		'Set viewpoint To the third quarter (Left bottom)
		driver.SetViewport(_RECTI(0,ResY/2,ResX/2,ResY))
		'Draw scene
		smgr.drawAll()
		
		'Set viewport the last quarter (Right bottom)
		driver.SetViewport(_RECTI(ResX/2,ResY/2,ResX,ResY))
	EndIf
	
	'Activate camera4
	smgr.setActiveCamera(camera[3])
	'Draw scene
	smgr.drawAll()

	driver.endScene()

	Rem
	As you can probably see, the image is rendered For every
	viewport seperately. That means, that you'll loose much performance.
	Ok, If you're aksing "How do I have to set the viewport
	To get this Or that screen?", don't panic. It's really
	easy: In the rect-Function you define 4 coordinates:
	- X-coordinate of the corner Left top
	- Y-coordinate of the corner Left top
	- X-coordinate of the corner Right bottom
	- Y-coordinate of the corner Right bottom

	That means, If you want To split the screen into 2 viewports
	you would give the following coordinates:
	- 1st viewport: 0,0,ResX/2,ResY
	- 2nd viewport: ResX/2,0,ResX,ResY

	If you didn't fully understand, just play arround with the example
	To check out what happens.

	Now we just view the current fps And shut down the engine,
	when the user wants To:
	EndRem
	'Get And show fps
	fps = driver.getFPS()
	If (lastFPS <> fps) Then
		str$ = "Irrlicht SplitScreen-Example (FPS: "+fps+")"

		device.setWindowCaption(str$)
		lastFPS = fps
	EndIf
Wend

'Delete device
device.drop()

Rem
That's it! Just compile and play around with the program.
Note: With the S-Key you can switch between using splitscreen
And Not.
EndRem

